#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.django.rattail.models.contact`` -- Models for 'contact' Extension
"""

from __future__ import absolute_import

from django.db import models

from rattail.django.rattail.models import Model, uuid_field


__all__ = ['PhoneNumber', 'EmailAddress']


class ContactInfoManager(models.Manager):

    def __init__(self, parent_type):
        super(ContactInfoManager, self).__init__()
        self.parent_type = parent_type

    def get_query_set(self):
        qs = super(ContactInfoManager, self).get_query_set()
        qs = qs.filter(parent_type=self.parent_type)
        qs = qs.order_by('preference')
        return qs


class ContactInfo(Model):

    class Meta(Model.Meta):
        abstract = True

    uuid = uuid_field()
    parent_type = models.CharField(max_length=20)
    preference = models.IntegerField()
    type = models.CharField(max_length=15, blank=True, null=True)


class PhoneNumber(ContactInfo):
    """
    Represents a phone (or fax) number associated with a contactable entity.
    """

    class Meta(ContactInfo.Meta):
        abstract = True
        db_table = ContactInfo.prefix('phone_numbers')

    number = models.CharField(max_length=20)

    def __repr__(self):
        return "<%s: %s>" % (self.__class__.__name__, self.number)

    def __unicode__(self):
        return unicode(self.number)


class EmailAddress(ContactInfo):
    """
    Represents an email address associated with a contactable entity.
    """

    class Meta(ContactInfo.Meta):
        abstract = True
        db_table = ContactInfo.prefix('email_addresses')

    address = models.CharField(max_length=255)

    def __repr__(self):
        return "<%s: %s>" % (self.__class__.__name__, self.address)

    def __unicode__(self):
        return unicode(self.address)
